@@ -23,9 +23,7 @@
  *   Boston, MA 02110-1301 USA
  */
 
-#ifdef HAVE_CONFIG_H
-#include <config.h>
-#endif
+#include "nm-default.h"
 
 #include <string.h>
 #include <errno.h>
@@ -37,17 +35,10 @@
 #include <libxml/tree.h>
 
 #include <gtk/gtk.h>
-#include <glib/gi18n.h>
 #include <glib-unix.h>
 
-#include <NetworkManager.h>
-#include <nm-vpn-service-plugin.h>
-
-#define SECRET_API_SUBJECT_TO_CHANGE
 #include <libsecret/secret.h>
 
-#include "../src/nm-openconnect-service-defines.h"
-
 #include "openconnect.h"
 
 #if !OPENCONNECT_CHECK_VER(2,1)
@@ -91,6 +82,8 @@
 #define write_config_const		/* */
 #endif
 
+#define AUTOSUBMIT_LIMIT 5
+
 static const SecretSchema openconnect_secret_schema = {
 	"org.freedesktop.NetworkManager.Connection.Openconnect",
 	SECRET_SCHEMA_DONT_MATCH_NAME,
@@ -174,8 +167,13 @@ typedef struct auth_ui_data {
 	GtkWidget *cancel_button;
 	GtkWidget *login_button;
 	GtkWidget *last_notice_icon;
+	GtkWidget *autoconnect;
+
 	GtkTextBuffer *log;
 
+	oc_token_mode_t token_mode;
+	const char *token_secret;
+
 	int cookie_retval;
 
 	int cancel_pipes[2];
@@ -197,6 +195,9 @@ typedef struct auth_ui_data {
 
 	GCond cert_response_changed;
 	enum certificate_response cert_response;
+
+	int autosubmit;
+	int fields_pending;
 } auth_ui_data;
 
 enum {
@@ -452,6 +453,15 @@ static char *find_form_answer(GHashTable *secrets, struct oc_auth_form *form,
 	return result;
 }
 
+static void form_autosubmit(auth_ui_data *ui_data)
+{
+	if (ui_data->fields_pending == 0 && ui_data->autosubmit) {
+		ui_data->autosubmit--;
+		gtk_button_clicked (GTK_BUTTON(ui_data->login_button));
+		gtk_widget_set_sensitive (ui_data->login_button, FALSE);
+	}
+}
+
 /* Callback which is called when we got a reply from the secret store for any
  * password field. Updates the contents of the password field unless the user
  * entered anything in the meantime. */
@@ -463,6 +473,7 @@ static void got_keyring_pw(GObject *object, GAsyncResult *result, gpointer userd
 	SecretValue *value = NULL;
 	const char *string = NULL;
 
+	data->ui_data->fields_pending--;
 	list = secret_service_search_finish (SECRET_SERVICE (object), result, NULL);
 	if (list != NULL) {
 		item = list->data;
@@ -480,7 +491,8 @@ static void got_keyring_pw(GObject *object, GAsyncResult *result, gpointer userd
 			}
 		} else
 			data->entry_text = g_strdup (string);
-	}
+	} else
+		data->ui_data->autosubmit = 0;
 
 	if (value)
 		secret_value_unref (value);
@@ -489,6 +501,8 @@ static void got_keyring_pw(GObject *object, GAsyncResult *result, gpointer userd
 	/* zero the cancellable so that we don’t attempt to cancel it when
 	 * closing the dialog */
 	g_clear_object (&data->cancel);
+
+	form_autosubmit (data->ui_data);
 }
 
 /* This part for processing forms from openconnect directly, rather than
@@ -533,8 +547,10 @@ static gboolean ui_form (struct oc_auth_form *form)
 			if (opt->type != OC_FORM_OPT_PASSWORD) {
 				data->entry_text = g_strdup (find_form_answer(ui_data->secrets,
 									      form, opt));
-				if (!data->entry_text)
+				if (!data->entry_text) {
 					data->entry_text = dup_option_value(opt);
+					ui_data->autosubmit = 0;
+				}
 			} else {
 				GHashTable *attrs;
 
@@ -544,6 +560,7 @@ static gboolean ui_form (struct oc_auth_form *form)
 				                                 "auth_id", form->auth_id,
 				                                 "label", data->opt->name,
 				                                 NULL);
+				ui_data->fields_pending++;
 				secret_service_search (NULL, &openconnect_secret_schema, attrs,
 				                       SECRET_SEARCH_ALL | SECRET_SEARCH_UNLOCK | SECRET_SEARCH_LOAD_SECRETS,
 				                       data->cancel, got_keyring_pw, data);
@@ -557,6 +574,8 @@ static gboolean ui_form (struct oc_auth_form *form)
 			g_mutex_unlock (&ui_data->form_mutex);
 			data->entry_text = g_strdup (find_form_answer(ui_data->secrets,
 								      form, opt));
+			if (!data->entry_text)
+				ui_data->autosubmit = 0;
 
 			if (opt == AUTHGROUP_OPT(form))
 				data->initial_selection = AUTHGROUP_SELECTION(form);
@@ -567,6 +586,8 @@ static gboolean ui_form (struct oc_auth_form *form)
 		} else
 			g_slice_free (ui_fragment_data, data);
 	}
+
+	form_autosubmit (ui_data);
 	
 	return ui_show(ui_data);
 }
@@ -688,7 +709,7 @@ static int nm_process_auth_form (void *cbdata, struct oc_auth_form *form)
 static char* get_title(const char *vpn_name)
 {
 	if (vpn_name)
-		return g_strdup_printf(_("Connect to VPN '%s'"), vpn_name);
+		return g_strdup_printf(_("Connect to VPN “%s”"), vpn_name);
 	else
 		return g_strdup(_("Connect to VPN"));
 }
@@ -730,7 +751,7 @@ static gboolean user_validate_cert(cert_data *data)
 	title = get_title(data->ui_data->vpn_name);
 	dlg = gtk_message_dialog_new(NULL, 0, GTK_MESSAGE_QUESTION,
 				     GTK_BUTTONS_OK_CANCEL,
-	                             _("Certificate from VPN server \"%s\" failed verification.\n"
+	                             _("Certificate from VPN server “%s” failed verification.\n"
 			             "Reason: %s\nDo you want to accept it?"),
 			             openconnect_get_hostname(data->ui_data->vpninfo),
 			             data->reason);
@@ -803,6 +824,8 @@ static int validate_peer_cert(void *cbdata,
 	if (accepted_hash && !openconnect_check_peer_cert_hash(ui_data->vpninfo, accepted_hash))
 		goto accepted;
 
+	ui_data->autosubmit = 0;
+
 	data = g_slice_new(cert_data);
 	data->ui_data = ui_data; /* FIXME uses global */
 	data->cert_details = openconnect_get_peer_cert_details(ui_data->vpninfo);
@@ -929,9 +952,10 @@ static int parse_xmlconfig(gchar *xmlconfig)
 	return 0;
 }
 
-static int get_config (GHashTable *options, GHashTable *secrets,
-		       struct openconnect_info *vpninfo)
+static int get_config (auth_ui_data *ui_data,
+		       GHashTable *options, GHashTable *secrets)
 {
+	struct openconnect_info *vpninfo = ui_data->vpninfo;
 	char *proxy;
 	char *xmlconfig;
 	char *hostname;
@@ -943,6 +967,7 @@ static int get_config (GHashTable *options, GHashTable *secrets,
 	char *cafile;
 	char *token_mode;
 	char *token_secret;
+	char *protocol;
 
 	hostname = g_hash_table_lookup (options, NM_OPENCONNECT_KEY_GATEWAY);
 	if (!hostname) {
@@ -985,6 +1010,14 @@ static int get_config (GHashTable *options, GHashTable *secrets,
 		parse_xmlconfig (config_str);
 	}
 
+	protocol = g_hash_table_lookup (options, NM_OPENCONNECT_KEY_PROTOCOL);
+	if (protocol && strcmp(protocol, "anyconnect")) {
+#if OPENCONNECT_CHECK_VER(5,1)
+		if (openconnect_set_protocol(vpninfo, protocol))
+#endif
+			return -EINVAL;
+	}
+
 	cafile = g_hash_table_lookup (options, NM_OPENCONNECT_KEY_CACERT);
 	if (cafile)
 		openconnect_set_cafile(vpninfo, OC3DUP (cafile));
@@ -1030,7 +1063,17 @@ static int get_config (GHashTable *options, GHashTable *secrets,
 		else if (!strcmp(token_mode, "hotp") && token_secret)
 			ret = __openconnect_set_token_mode(vpninfo, OC_TOKEN_MODE_HOTP, token_secret);
 #endif
-
+#if OPENCONNECT_CHECK_VER(5,0)
+		else if (!strcmp(token_mode, "yubioath")) {
+			/* This needs to be done from a thread because it can call back to
+			   ask for the PIN */
+			ui_data->token_mode = OC_TOKEN_MODE_YUBIOATH;
+			if (token_secret && token_secret[0])
+				ui_data->token_secret = token_secret;
+			else
+				ui_data->token_secret = NULL;
+		}
+#endif
 		if (ret)
 			fprintf(stderr, "Failed to initialize software token: %d\n", ret);
 	}
@@ -1144,6 +1187,7 @@ static gboolean write_notice_real(char *message)
 }
 
 /* runs in worker thread */
+ _nm_printf (3, 4)
 static void write_progress(void *cbdata, int level, const char *fmt, ...)
 {
 	va_list args;
@@ -1179,6 +1223,8 @@ static gboolean cookie_obtained(auth_ui_data *ui_data)
 	ui_data->getting_cookie = FALSE;
 	gtk_widget_hide (ui_data->getting_form_label);
 
+	ui_data->autosubmit = 0;
+
 	if (ui_data->cancelled) {
 		/* user has chosen a new host, start from beginning */
 		g_hash_table_remove_all (ui_data->success_secrets);
@@ -1419,7 +1465,9 @@ static void build_main_dialog(auth_ui_data *ui_data)
 			gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autocon), 1);
 		g_signal_connect(autocon, "toggled", G_CALLBACK(autocon_toggled), NULL);
 		gtk_widget_show(autocon);
+		ui_data->autoconnect = autocon;
 	}
+
 	frame = gtk_frame_new(NULL);
 	gtk_box_pack_start(GTK_BOX(vbox), frame, TRUE, TRUE, 0);
 	gtk_widget_set_size_request(frame, -1, -1);
@@ -1435,7 +1483,7 @@ static void build_main_dialog(auth_ui_data *ui_data)
 	gtk_box_pack_start(GTK_BOX(frame_box), ui_data->no_form_label, FALSE, FALSE, 0);
 	gtk_widget_show(ui_data->no_form_label);
 
-	ui_data->getting_form_label = gtk_label_new(_("Contacting host, please wait..."));
+	ui_data->getting_form_label = gtk_label_new(_("Contacting host, please wait…"));
 	gtk_widget_set_sensitive(ui_data->getting_form_label, FALSE);
 	gtk_box_pack_start(GTK_BOX(frame_box), ui_data->getting_form_label, FALSE, FALSE, 0);
 
@@ -1511,6 +1559,8 @@ static auth_ui_data *init_ui_data (char *vpn_name, GHashTable *options, GHashTab
 							  g_free, g_free);
 	ui_data->success_passwords = g_hash_table_new_full (g_str_hash, g_str_equal,
 							  g_free, keyring_password_free);
+	ui_data->autosubmit = AUTOSUBMIT_LIMIT;
+
 	if (pipe(ui_data->cancel_pipes)) {
 		/* This should never happen, and the world is probably about
 		   to come crashing down around our ears. But attempt to cope
@@ -1570,12 +1620,30 @@ static struct option long_options[] = {
 	{NULL, 0, 0, 0},
 };
 
+static gpointer init_connection (auth_ui_data *ui_data)
+{
+	if (ui_data->token_mode != OC_TOKEN_MODE_NONE)
+		__openconnect_set_token_mode(ui_data->vpninfo, ui_data->token_mode, ui_data->token_secret);
+
+	gtk_widget_set_sensitive (ui_data->combo, TRUE);
+	gtk_widget_set_sensitive (ui_data->connect_button, TRUE);
+
+	/* Start connecting now if there's only one host. Or if configured to */
+	if (!vpnhosts->next || gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (ui_data->autoconnect)))
+		queue_connect_host(_ui_data);
+	else
+		ui_data->autosubmit = 0;
+
+	return NULL;
+}
+
 int main (int argc, char **argv)
 {
 	char *vpn_name = NULL, *vpn_uuid = NULL, *vpn_service = NULL;
 	GHashTable *options = NULL, *secrets = NULL;
 	gboolean allow_interaction = FALSE;
 	GHashTableIter iter;
+	GThread *init_thread;
 	gchar *key, *value;
 	int opt;
 
@@ -1638,7 +1706,7 @@ int main (int argc, char **argv)
 	gtk_init(0, NULL);
 
 	_ui_data = init_ui_data(vpn_name, options, secrets, vpn_uuid);
-	if (get_config(options, secrets, _ui_data->vpninfo)) {
+	if (get_config(_ui_data, options, secrets)) {
 		fprintf(stderr, "Failed to find VPN UUID %s\n", vpn_uuid);
 		return 1;
 	}
@@ -1651,19 +1719,24 @@ int main (int argc, char **argv)
 
 	openconnect_init_ssl();
 
-	/* Start connecting now if there's only one host. Or if configured to */
-	if (!vpnhosts->next || get_autoconnect (secrets))
-		queue_connect_host(_ui_data);
+	/* These can't be done until token password handled */
+	gtk_widget_set_sensitive (_ui_data->combo, FALSE);
+	gtk_widget_set_sensitive (_ui_data->connect_button, FALSE);
+
+	init_thread = g_thread_new("init_connection", (GThreadFunc)init_connection, _ui_data);
+	g_thread_unref(init_thread);
 
 	gtk_window_present(GTK_WINDOW(_ui_data->dialog));
 	gtk_main();
 
+	if (!g_hash_table_size (_ui_data->secrets))
+		return 0;
+
 	/* Dump all secrets to stdout */
 	g_hash_table_iter_init (&iter, _ui_data->secrets);
 	while (g_hash_table_iter_next (&iter, (gpointer *)&key,
 				       (gpointer *)&value))
 		printf("%s\n%s\n", key, value);
-
 	printf("\n\n");
 	fflush(stdout);
 
