@@ -22,7 +22,8 @@
  *   Copyright © 2007 - 2008 Novell, Inc.
  */
 
-#include <glib.h>
+#include "nm-default.h"
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -30,24 +31,56 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 #include <errno.h>
-#include <NetworkManager.h>
 
-#include "nm-openconnect-service.h"
-#include "nm-utils.h"
+#include "nm-utils/nm-shared-utils.h"
+#include "nm-utils/nm-vpn-plugin-macros.h"
+
+extern char **environ;
+
+/*****************************************************************************/
+
+static struct {
+	int log_level;
+	const char *log_prefix_token;
+} gl/*obal*/;
+
+/*****************************************************************************/
+
+#define _NMLOG(level, ...) \
+	G_STMT_START { \
+		if (gl.log_level >= (level)) { \
+			g_print ("nm-openconnect[%s] %-7s [helper-%ld] " _NM_UTILS_MACRO_FIRST (__VA_ARGS__) "\n", \
+			         gl.log_prefix_token, \
+			         nm_utils_syslog_to_str (level), \
+			         (long) getpid () \
+			         _NM_UTILS_MACRO_REST (__VA_ARGS__)); \
+		} \
+	} G_STMT_END
+
+static gboolean
+_LOGD_enabled (void)
+{
+	return gl.log_level >= LOG_INFO;
+}
+
+#define _LOGD(...) _NMLOG(LOG_INFO,    __VA_ARGS__)
+#define _LOGW(...) _NMLOG(LOG_WARNING, __VA_ARGS__)
+
+/*****************************************************************************/
 
 static void
 helper_failed (GDBusProxy *proxy, const char *reason)
 {
 	GError *err = NULL;
 
-	g_warning ("nm-nopenconnect-service-openconnect-helper did not receive a valid %s from openconnect", reason);
+	_LOGW ("nm-nopenconnect-service-openconnect-helper did not receive a valid %s from openconnect", reason);
 
 	if (!g_dbus_proxy_call_sync (proxy, "SetFailure",
 	                             g_variant_new ("(s)", reason),
 	                             G_DBUS_CALL_FLAGS_NONE, -1,
 	                             NULL,
 	                             &err)) {
-		g_warning ("Could not send failure information: %s", err->message);
+		_LOGW ("Could not send failure information: %s", err->message);
 		g_error_free (err);
 	}
 
@@ -87,7 +120,7 @@ send_config (GDBusProxy *proxy, GVariant *config,
 
 	return;
 error:
-	g_warning ("Could not send configuration information: %s", err->message);
+	_LOGW ("Could not send configuration information: %s", err->message);
 	g_error_free (err);
 }
 
@@ -218,16 +251,36 @@ addr6_list_to_gvariant (const char *str)
 static GVariant *
 split_dns_list_to_gvariant (const char *str)
 {
+	GVariant *var = NULL;
 	gchar **split;
+	int i, j;
 
 	if (!str || strlen (str) < 1)
 		return NULL;
 
-	split = g_strsplit (str, ",", -1);
-	if (g_strv_length (split) == 0)
+	split = g_strsplit_set (str, ", ", -1);
+	if (!split)
 		return NULL;
 
-	return g_variant_new_strv ((const gchar **) split, -1);
+	/* Eliminate empty strings */
+	for (i = 0, j = 0; split[i]; i++) {
+		if (split[i][0]) {
+			if (j != i) {
+				split[j] = split[i];
+				split[i] = NULL;
+			}
+			j++;
+		} else {
+			g_free(split[i]);
+			split[i] = NULL;
+		}
+	}
+
+	if (j)
+		var = g_variant_new_strv ((const gchar **)split, -1);
+	g_strfreev (split);
+
+	return var;
 }
 
 static GVariant *
@@ -260,7 +313,7 @@ get_ip4_routes (void)
 		snprintf (buf, BUFLEN, "CISCO_SPLIT_INC_%d_ADDR", i);
 		tmp = getenv (buf);
 		if (!tmp || inet_pton (AF_INET, tmp, &network) <= 0) {
-			g_warning ("Ignoring invalid static route address '%s'", tmp ? tmp : "NULL");
+			_LOGW ("Ignoring invalid static route address '%s'", tmp ? tmp : "NULL");
 			continue;
 		}
 
@@ -272,7 +325,7 @@ get_ip4_routes (void)
 			errno = 0;
 			tmp_prefix = strtol (tmp, NULL, 10);
 			if (errno || tmp_prefix <= 0 || tmp_prefix > 32) {
-				g_warning ("Ignoring invalid static route prefix '%s'", tmp ? tmp : "NULL");
+				_LOGW ("Ignoring invalid static route prefix '%s'", tmp ? tmp : "NULL");
 				continue;
 			}
 			prefix = (guint32) tmp_prefix;
@@ -282,7 +335,7 @@ get_ip4_routes (void)
 			snprintf (buf, BUFLEN, "CISCO_SPLIT_INC_%d_MASK", i);
 			tmp = getenv (buf);
 			if (!tmp || inet_pton (AF_INET, tmp, &netmask) <= 0) {
-				g_warning ("Ignoring invalid static route netmask '%s'", tmp ? tmp : "NULL");
+				_LOGW ("Ignoring invalid static route netmask '%s'", tmp ? tmp : "NULL");
 				continue;
 			}
 			prefix = nm_utils_ip4_netmask_to_prefix (netmask.s_addr);
@@ -334,7 +387,7 @@ get_ip6_routes (void)
 		snprintf (buf, BUFLEN, "CISCO_IPV6_SPLIT_INC_%d_ADDR", i);
 		network = getenv (buf);
 		if (!network) {
-			g_warning ("Ignoring invalid static route address '%s'", network ? network : "NULL");
+			_LOGW ("Ignoring invalid static route address '%s'", network ? network : "NULL");
 			continue;
 		}
 
@@ -346,18 +399,18 @@ get_ip6_routes (void)
 			errno = 0;
 			tmp_prefix = strtol (tmp, NULL, 10);
 			if (errno || tmp_prefix <= 0 || tmp_prefix > 128) {
-				g_warning ("Ignoring invalid static route prefix '%s'", tmp ? tmp : "NULL");
+				_LOGW ("Ignoring invalid static route prefix '%s'", tmp ? tmp : "NULL");
 				continue;
 			}
 			prefix = (guint32) tmp_prefix;
 		} else {
-			g_warning ("Ignoring static route %d with no prefix length", i);
+			_LOGW ("Ignoring static route %d with no prefix length", i);
 			continue;
 		}
 
 		route = nm_ip_route_new (AF_INET6, network, prefix, NULL, -1, &error);
 		if (!route) {
-			g_warning ("Ignoring a route: %s", error->message);
+			_LOGW ("Ignoring a route: %s", error->message);
 			g_error_free (error);
 			continue;
 		}
@@ -382,10 +435,11 @@ get_ip6_routes (void)
  * INTERNAL_IP4_DNS       -- list of dns serverss
  * INTERNAL_IP4_NBNS      -- list of wins servers
  * CISCO_DEF_DOMAIN       -- default domain name
+ * CISCO_SPLIT_DNS        -- default domain name
  * CISCO_BANNER           -- banner from server
  *
  */
-int 
+int
 main (int argc, char *argv[])
 {
 	GDBusProxy *proxy;
@@ -396,11 +450,38 @@ main (int argc, char *argv[])
 	GError *err = NULL;
 	struct in_addr temp_addr;
 	char *bus_path;
+	gboolean has_ip4 = FALSE, has_ip6 = FALSE;
 
 #if !GLIB_CHECK_VERSION (2, 35, 0)
 	g_type_init ();
 #endif
 
+	gl.log_level = _nm_utils_ascii_str_to_int64 (getenv ("NM_VPN_LOG_LEVEL"),
+	                                             10, 0, LOG_DEBUG,
+	                                             LOG_NOTICE);
+	gl.log_prefix_token = getenv ("NM_VPN_LOG_PREFIX_TOKEN") ?: "???";
+
+	if (_LOGD_enabled ()) {
+		GString *args;
+		const char **iter;
+		guint i;
+
+		args = g_string_new (NULL);
+		for (i = 0; i < argc; i++) {
+			if (i > 0)
+				g_string_append_c (args, ' ');
+			tmp = g_strescape (argv[i], NULL);
+			g_string_append_printf (args, "\"%s\"", tmp);
+			g_free (tmp);
+		}
+
+		_LOGD ("command line: %s", args->str);
+		g_string_free (args, TRUE);
+
+		for (iter = (const char **) environ; iter && *iter; iter++)
+			_LOGD ("environment: %s", *iter);
+	}
+
 	/* openconnect gives us a "reason" code.  If we are given one,
 	 * don't proceed unless its "connect".
 	 */
@@ -420,7 +501,7 @@ main (int argc, char *argv[])
 	                                       NM_VPN_DBUS_PLUGIN_INTERFACE,
 	                                       NULL, &err);
 	if (!proxy) {
-		g_warning ("Could not create a D-Bus proxy: %s", err->message);
+		_LOGW ("Could not create a D-Bus proxy: %s", err->message);
 		g_error_free (err);
 		exit (1);
 	}
@@ -450,10 +531,10 @@ main (int argc, char *argv[])
 	if (val)
 		g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_CONFIG_BANNER, val);
 
-	/* Default domain */
-	val = str_to_gvariant (getenv ("CISCO_DEF_DOMAIN"), TRUE);
+	/* Proxy */
+	val = str_to_gvariant (getenv ("CISCO_PROXY_PAC"), TRUE);
 	if (val)
-		g_variant_builder_add (&ip4builder, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_DOMAIN, val);
+		g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_CONFIG_PROXY_PAC, val);
 
 	/* MTU  */
 	tmp = getenv ("INTERNAL_IP4_MTU");
@@ -463,7 +544,7 @@ main (int argc, char *argv[])
 		errno = 0;
 		mtu = strtol (tmp, NULL, 10);
 		if (errno || mtu < 0 || mtu > 20000) {
-			g_warning ("Ignoring invalid tunnel MTU '%s'", tmp);
+			_LOGW ("Ignoring invalid tunnel MTU '%s'", tmp);
 		} else {
 			val = g_variant_new_uint32 ((guint32) mtu);
 			g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_CONFIG_MTU, val);
@@ -471,18 +552,25 @@ main (int argc, char *argv[])
 	}
 
 	/* IPv4 address */
-	val = addr4_to_gvariant (getenv ("INTERNAL_IP4_ADDRESS"));
-	if (val)
-		g_variant_builder_add (&ip4builder, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_ADDRESS, val);
-	else
-		helper_failed (proxy, "IP4 Address");
+	tmp = getenv ("INTERNAL_IP4_ADDRESS");
+	if (tmp && strlen (tmp)) {
+		val = addr4_to_gvariant (tmp);
+		if (val)
+			g_variant_builder_add (&ip4builder, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_ADDRESS, val);
+		else
+			helper_failed (proxy, "IP4 Address");
+		has_ip4 = TRUE;
+	}
 
 	/* IPv4 PTP address; for openconnect PTP address == internal IPv4 address */
-	val = addr4_to_gvariant (getenv ("INTERNAL_IP4_ADDRESS"));
-	if (val)
-		g_variant_builder_add (&ip4builder, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_PTP, val);
-	else
-		helper_failed (proxy, "IP4 PTP Address");
+	tmp = getenv ("INTERNAL_IP4_ADDRESS");
+	if (tmp && strlen (tmp)) {
+		val = addr4_to_gvariant (tmp);
+		if (val)
+			g_variant_builder_add (&ip4builder, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_PTP, val);
+		else
+			helper_failed (proxy, "IP4 Address");
+	}
 
 	/* IPv4 Netmask */
 	tmp = getenv ("INTERNAL_IP4_NETMASK");
@@ -501,10 +589,27 @@ main (int argc, char *argv[])
 	if (val)
 		g_variant_builder_add (&ip4builder, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_NBNS, val);
 
-	/* Split DNS domains */
+	/* We have two environment variables with domains --
+	   CISCO_SPLIT_DNS and CISCO_DEF_DOMAIN. On Cisco,
+	   CISCO_DEF_DOMAIN can only be a single domain, while
+	   CISCO_SPLIT_DNS can have multiple domains separated by
+	   comma. On Juniper, CISCO_SPLIT_DNS is not supported but
+	   CISCO_DEF_DOMAIN can have multiple domains separated by ", ".
+
+	   The upshot of all this is we use CISCO_SPLIT_DNS if available,
+	   CISCO_DEF_DOMAIN if not. */
+
 	val = split_dns_list_to_gvariant (getenv ("CISCO_SPLIT_DNS"));
-	if (val)
-		g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_IP4_CONFIG_DOMAINS, val);
+	if (val) {
+		g_variant_builder_add (&ip4builder, "{sv}",
+				       NM_VPN_PLUGIN_IP4_CONFIG_DOMAINS, val);
+	} else {
+		val = split_dns_list_to_gvariant (getenv ("CISCO_DEF_DOMAIN"));
+		if (val) {
+			g_variant_builder_add (&ip4builder, "{sv}",
+					       NM_VPN_PLUGIN_IP4_CONFIG_DOMAINS, val);
+		}
+	}
 
 	/* Routes */
 	val = get_ip4_routes ();
@@ -515,6 +620,11 @@ main (int argc, char *argv[])
 		                       g_variant_new_boolean (TRUE));
 	}
 
+	/* Default domain */
+	val = str_to_gvariant (getenv ("CISCO_DEF_DOMAIN"), TRUE);
+	if (val)
+		g_variant_builder_add (&ip6builder, "{sv}", NM_VPN_PLUGIN_IP6_CONFIG_DOMAIN, val);
+
 	/* IPv6 address */
 	tmp = getenv ("INTERNAL_IP6_ADDRESS");
 	if (tmp && strlen (tmp)) {
@@ -523,6 +633,7 @@ main (int argc, char *argv[])
 			g_variant_builder_add (&ip6builder, "{sv}", NM_VPN_PLUGIN_IP6_CONFIG_ADDRESS, val);
 		else
 			helper_failed (proxy, "IP6 Address");
+		has_ip6 = TRUE;
 	}
 
 	/* IPv6 PTP address; for openconnect PTP address == internal IPv6 address */
@@ -560,7 +671,7 @@ main (int argc, char *argv[])
 
 	ip4config = g_variant_builder_end (&ip4builder);
 
-	if (g_variant_n_children (ip4config)) {
+	if (has_ip4) {
 		val = g_variant_new_boolean (TRUE);
 		g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_CONFIG_HAS_IP4, val);
 	} else {
@@ -570,7 +681,7 @@ main (int argc, char *argv[])
 
 	ip6config = g_variant_builder_end (&ip6builder);
 
-	if (g_variant_n_children (ip6config)) {
+	if (has_ip6) {
 		val = g_variant_new_boolean (TRUE);
 		g_variant_builder_add (&builder, "{sv}", NM_VPN_PLUGIN_CONFIG_HAS_IP6, val);
 	} else {
